import 'dart:async';

import 'package:eosdart/eosdart.dart';
import 'package:flutter/material.dart' hide Action;
import 'package:seeds/providers/services/eos_service.dart';
import 'package:seeds/widgets/broadcast_transaction_overlay.dart';
import 'package:seeds/widgets/main_button.dart';
import 'package:seeds/widgets/transaction_details.dart';
import 'package:seeds/i18n/wallet.i18n.dart';

class CustomTransactionArguments {
  final String? account;
  final String? name;
  final Map<String, dynamic>? data;

  CustomTransactionArguments({this.account, this.name, this.data});
}

class CustomTransaction extends StatefulWidget {
  final CustomTransactionArguments arguments;

  const CustomTransaction(this.arguments);

  @override
  _CustomTransactionState createState() => _CustomTransactionState();
}

class _CustomTransactionState extends State<CustomTransaction> {
  final plantController = TextEditingController(text: '1');

  bool transactionSubmitted = false;

  final StreamController<bool> _statusNotifier = StreamController<bool>.broadcast();
  final StreamController<String?> _messageNotifier = StreamController<String?>.broadcast();

  @override
  void dispose() {
    _statusNotifier.close();
    _messageNotifier.close();
    super.dispose();
  }

  void onSubmit() async {
    setState(() {
      transactionSubmitted = true;
    });

    try {
      String? transactionId = await sendTransaction();

      _statusNotifier.add(true);
      _messageNotifier.add(transactionId);
    } catch (err) {
      _statusNotifier.add(false);
      _messageNotifier.add(err.toString());
    }
  }

  Widget buildProgressOverlay() {
    return BroadcastTransactionOverlay(
      statusStream: _statusNotifier.stream,
      messageStream: _messageNotifier.stream,
      onClose: () {
        Navigator.of(context).maybePop();
      },
    );
  }

  Widget buildTransactionForm() {
    var name = widget.arguments.name;
    var account = widget.arguments.account;
    var data = widget.arguments.data!;

    return Scaffold(
      appBar: AppBar(
        leading: IconButton(
          icon: const Icon(Icons.close, color: Colors.black),
          onPressed: () {
            Navigator.pop(context);
          },
        ),
        backgroundColor: Colors.transparent,
        elevation: 0,
      ),
      backgroundColor: Colors.white,
      body: Container(
        margin: const EdgeInsets.symmetric(horizontal: 17),
        child: SingleChildScrollView(
          child: Column(
            children: <Widget>[
              TransactionDetails(
                image: Image.asset(
                  'assets/images/seeds.png',
                  height: 33,
                ),
                title: name,
                beneficiary: account,
              ),
              const SizedBox(height: 8),
              Column(
                children: <Widget>[
                  ...data.entries
                      .map(
                        (e) => Row(
                          mainAxisAlignment: MainAxisAlignment.spaceBetween,
                          children: <Widget>[
                            Text(
                              e.key,
                              style: const TextStyle(
                                fontFamily: "heebo",
                                fontSize: 18,
                                color: Colors.black,
                                fontWeight: FontWeight.w400,
                              ),
                            ),
                            Text(
                              e.value.toString(),
                              style: const TextStyle(
                                fontFamily: "heebo",
                                fontSize: 18,
                                color: Colors.black,
                                fontWeight: FontWeight.w400,
                              ),
                            ),
                          ],
                        ),
                      )
                      .toList(),
                ],
              ),
              const SizedBox(height: 8),
              MainButton(
                margin: const EdgeInsets.only(top: 25),
                title: 'Send'.i18n,
                onPressed: onSubmit,
              ),
            ],
          ),
        ),
      ),
    );
  }

  Future<String?> sendTransaction() async {
    var name = widget.arguments.name;
    var account = widget.arguments.account;
    var data = widget.arguments.data;

    var response = await EosService.of(context, listen: false).sendTransaction([
      Action()
        ..account = account
        ..name = name
        ..data = data
    ]);

    return response["transaction_id"];
  }

  @override
  Widget build(BuildContext context) {
    return Stack(
      children: <Widget>[
        buildTransactionForm(),
        transactionSubmitted ? buildProgressOverlay() : Container(),
      ],
    );
  }
}
